# -*- coding: utf-8 -*-
# Copyright (C) 2018-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Package for supported Greenbone Protocol versions.

Currently `GMP version 20.08`_, `GMP version 21.04`_
`GMP version 22.04` and `OSP version 1`_ are supported.

.. _GMP version 20.08:
    https://docs.greenbone.net/API/GMP/gmp-20.08.html
.. _GMP version 21.04:
    https://docs.greenbone.net/API/GMP/gmp-21.04.html
.. _GMP version 22.04:
    https://docs.greenbone.net/API/GMP/gmp-22.04.html
.. _OSP version 1:
    https://docs.greenbone.net/API/OSP/osp-1.2.html
"""
